
/* miscellaneous defs */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>

char *findit(char *inpstr, FILE *fp);
char *findit_interactive(char *inpstr, char *filename);
char *convert(char *inpstr, char *key);
int lesser(char *a, char *b);

/* main program loop */

main(int argc, char *argv[])
{
FILE *fp;
int interactive=0;
char line[32];

if (argc<2) interactive=1;
else if (argc==2) fp=stdin;
else {
        /* check for validity of filename in argv[2] */
	if (!(fp=fopen(argv[2],"r")))
		exit(-1);
	}
if (argc>=2)
	if (*argv[1]=='-' && *(argv[1]+1)=='-' &&
	   (*(argv[1]+2)=='h' || *(argv[1]+2)=='H')) {
		printf("map_lookup [location [map_file]]\n");
		printf("where location is in hex, and may begin with 0x,\n");
		printf("and map_file is the name of the file containing\n");
		printf("a sorted list of kernel symbols (i.e. nm -n on\n");
		printf("an unstripped kernel).\n\n");
		printf("If map_file is not specified, standard input is\n");
		printf("assumed unless no arguments are given, in which\n");
		printf("case the program operates interactively.\n");
		exit(0);
		}

/* fp is guranteed to be a usable file pointer here */

if (interactive) imode();
else  {
	char *lineptr=argv[1];

	if (*argv[1]=='0' && *(argv[1]+1)=='x') { lineptr++; lineptr++; }
	printf("%s\n",lineptr=findit(lineptr,fp));
	if (fp!=stdin) fclose(fp);
	else while (1) if (fgets(line,30,fp)==NULL) break;
	}
exit(0);
}

imode()
{
char line[34], filename[1026];
char *lineptr;
FILE *fp;
int valid=0;

while (!valid) {
	printf("Enter filename.\n\r--> ");
	fgets(filename,1020,stdin);
	while ((filename[strlen(filename)-1]=='\n')
   		|| (filename[strlen(filename)-1]=='\r'))
		filename[strlen(filename)-1]=0;

	valid=1;
	if (!strlen(filename)) valid=0;
	else if (!(fp=fopen(filename,"r"))) valid=0;
	else fclose(fp);
	}
	
printf("Enter symbol to find or press return to end.");

while (1) {
	printf("\n\r--> ");
	if (fgets(line,32,stdin)==NULL) break;
	while ((line[strlen(line)-1]=='\n') || (line[strlen(line)-1]=='r'))
		line[strlen(line)-1]=0;
	if (!strlen(line)) break;
	lineptr=line;
	if (line[0]=='0' && line[1]=='x') { lineptr++; lineptr++; }
	printf("%s\n",lineptr=findit_interactive(lineptr,filename));
	}
}

/* misc. functions */

lesser(char *a, char *b)
{
long temp1, temp2;

temp1=strtol(a,(char **)NULL,16);
temp2=strtol(b,(char **)NULL,16);

if (temp1==temp2) return 0;
else if (temp1<temp2) return 1;
else return 2;
}
char *findit(char *inpstr, FILE *fp)
{
char key[34], line[1024], prevline[1024];
char *retval;
static char buffer[34];

line[0]=0;
prevline[0]=0;

retval=buffer;
sprintf(retval,"%s",inpstr);

sscanf(inpstr,"%s",key);

while (1) {
	if (fgets(line,1022,fp)==NULL) { return convert(prevline,key); }
	else if (lesser(line,key)==2) { return convert(prevline,key); }
	else sprintf(prevline,"%s",line);
	}
/* This shouldn't happen */
return NULL;
}

char *findit_interactive(char *inpstr, char *filename)
{
char key[34], line[1024], prevline[1024];
char *retval;
static char buffer[1026];
FILE *fp;

line[0]=0;
prevline[0]=0;

retval=buffer;
sprintf(buffer,"%s",inpstr);

if (!(fp=fopen(filename,"r"))) { error(1); return retval; }

sscanf(inpstr,"%s",key);

while (1) {
	if (fgets(line,1022,fp)==NULL) { fclose(fp); return convert(prevline,key); }
	else if (lesser(line,key)==2) { fclose(fp); return convert(prevline,key); }
	else sprintf(prevline,"%s",line);
	}
/* This shouldn't happen */
return NULL;
}



char *convert(char *inpstr,char *key)
{
unsigned int offset;
static char buffer[1024];
char *retval;

retval=buffer;

offset=strtol(key,(char **)NULL,16)-strtol(inpstr,(char **)NULL,16);

while ((inpstr[strlen(inpstr)-1]=='\r') || (inpstr[strlen(inpstr)-1]=='\n'))
	inpstr[strlen(inpstr)-1]=0;

sprintf(retval,"%s : offset %d",inpstr,offset);
return retval;
}

/* THE END */
